 %-------------------------------------------------------------------------
 % SECTION 4: Stochastic and Deterministic Trends, Decompositions
 %-------------------------------------------------------------------------
 % Select the estimation sample
s_start   = '31/01/1979';
s_end     = '29/10/1993';
date_find = datenum([s_start; s_end],'dd/mm/yyyy');
first        = datefind(date_find(1,1),date);
last        = datefind(date_find(2,1),date);
T=first-last+1;   
 % Generate artificial trend series
    Trend_det=NaN(size(GER.Stock.Ret));
    Trend=NaN(size(GER.Stock.Ret));
    Trend_st1=NaN(size(GER.Stock.Ret));
    Trend_st2=NaN(size(GER.Stock.Ret));
    Trend(1,1)=0;
    Trend_det(1,1)=0;
    Trend_st1(1,1)=0;
    Trend_st2(1,1)=0;
    ind=rows(Trend_st1);
    for i=2:ind
    Trend(i,:)=Trend(i-1,:)+1;
    Trend_det(i,:)=0.1*Trend(i,:)+normrnd(0,1,1);
    Trend_st1(i,:)=0.15+Trend_st1(i-1,:)+normrnd(0,1,1);
    Trend_st2(i,:)=0.1+Trend_st2(i-1,:)+normrnd(0,1,1);
    end
    % Plot trend series
    t=1:rows(GER.Stock.Ret);
    figure
    h3=plot(t',Trend_det,'.-',t',Trend_st1,'--',t',Trend_st2,'-','LineWidth',2);
    title('Trends','fontname','garamond','fontsize',16);
    set(gca,'fontname','garamond','fontsize',10);
    set(gca,'xtick',[1:24:rows(t')]); 
    set(gca,'xlim',[1 rows(t')]);
    set(gca,'xticklabel','1977|1979|1981|1983|1985|1987|1989|1991|1993|1995|1997|1999|2001|2003|2005|2007|2009|2011');
    grid;
    set(gcf,'color','w');
    h3=legend('Det Trend','ST TREND 1', 'ST TREND 2',2);
 
 
 %--------------------------------------------------------------------------
 % SECTION 5: Beveridge-Nelson Decomposition 
 %--------------------------------------------------------------------------
 % Generate artificial IMA series
    % Defining the parameters
    rho=0.6;
    theta=0.5;
    % Simulate IMA series
    IMA=NaN(size(GER.Stock.Ret));
    IMA(1,1)=0;
    IMA(2,1)=0;
    Num=rows(GER.Stock.Ret);
    eps=normrnd(0,1,Num,1);
    ind=rows(GER.Stock.Ret);
    for i=3:ind
   IMA(i,:)=IMA(i-1,:)+rho*(IMA(i-1,:)-IMA(i-2,:))+eps(i,:)+theta*eps(i-1,:);
    end
    % define BN decomposition 
    
    BN_TR=NaN(size(GER.Stock.Ret));
    BN_CY=NaN(size(GER.Stock.Ret));
    BN_TR(1,1)=0;
    for i=2:ind
   BN_TR(i,:)=BN_TR(i-1,:)+((1+theta)/(1-rho))*eps(i,:);
    end
    BN_CY=IMA-BN_TR;
    
    % Plot trend series
    t=1:rows(GER.Stock.Ret);
    figure
    h3=plot(t',(BN_TR+mean(BN_CY)),'.-',t',(BN_CY-mean(BN_CY)),'--',t',IMA,'-','LineWidth',2);
    title('Trends','fontname','garamond','fontsize',16);
    set(gca,'fontname','garamond','fontsize',10);
    set(gca,'xtick',[1:24:rows(t')]); 
    set(gca,'xlim',[1 rows(t')]);
    set(gca,'xticklabel','1977|1979|1981|1983|1985|1987|1989|1991|1993|1995|1997|1999|2001|2003|2005|2007|2009|2011');
    grid;
    set(gcf,'color','w');
    h3=legend('BN trend','BN cycle', 'IMA',2);
    
    [HP_TR,HP_CY] = hpfilter(IMA,14400);
    
    figure
    plot(t',(BN_TR+mean(BN_CY)),'.-',t',HP_TR,'--',t',IMA,'-','LineWidth',2);
    title('Trends','fontname','garamond','fontsize',16);
    set(gca,'fontname','garamond','fontsize',10);
    set(gca,'xtick',[1:24:rows(t')]); 
    set(gca,'xlim',[1 rows(t')]);
    set(gca,'xticklabel','1977|1979|1981|1983|1985|1987|1989|1991|1993|1995|1997|1999|2001|2003|2005|2007|2009|2011');
    grid;
    set(gcf,'color','w');
    h3=legend('BN trend','HP trend', 'IMA',2);
    
    