%--------------------------------------------------------------------------
% SECTION 5: Portfolio Allocation with univariate TS models 
%--------------------------------------------------------------------------
s_start   = '31/12/1984';
s_end     = '31/12/2003';
date_find = datenum([s_start; s_end],'dd/mm/yyyy');
ss        = datefind(date_find(1,1),date);
se        = datefind(date_find(2,1),date);
T=se-ss+1;
p_start = '30/01/2004';
p_end   = '31/12/2007';
date_find=datenum([p_start; p_end],'dd/mm/yyyy');
ps=datefind(date_find(1,1),date);
pe=datefind(date_find(2,1),date);
n = pe-ps+1;
%-----------  Historical Excess Returns Matrix ----------------------------
%% Step 1. Construct the relevant time series of returns 
Ret3Y = [GER.RF.Ret3Y(ss:se) GER.LTB.Ret3Y(ss:se) GER.Stock.Ret3Y(ss:se) UK.Stock.Reteu3Y(ss:se) US.Stock.Reteu3Y(ss:se)];
Ret = [GER.RiskFree(ss:se) GER.LTBond.Ret(ss:se) GER.Stock.Ret(ss:se) UK.Stock.Reteu(ss:se) US.Stock.Reteu(ss:se)];
ExRet= [GER.LTBond.ExRet(ss:se) GER.Stock.ExRet(ss:se) UK.Stock.ExRet(ss:se) US.Stock.ExRet(ss:se)];
ExRet3Y=[GER.LTB.ExRet3Y(ss:se) GER.Stock.ExRet3Y(ss:se) UK.Stock.ExRet3Y(ss:se) US.Stock.ExRet3Y(ss:se)];

% Descriptive data analysis
Perf = cumsum(Ret);
run Figure005 

%% ------------------------------------------------------------------------
% Compute the weights and Efficient Frontier using CER upto 2003
% -------------------------------------------------------------------------
% FOrecasting model specification and construction of the relevant measures
muExR    = mean(ExRet3Y);
SigmaR = cov(ExRet3Y); %compute robust variance covariance matrix
CorR=corr(ExRet3Y);
% Optimal weights on the tangency portfolio

WeightsTP    = ((SigmaR^(-1))*muExR')./(ones(size(SigmaR,2),1)'*(SigmaR^(-1)*muExR'));
%Compute the efficient frontier and plot it 
NumPortf = 20;  % number of efficient portfolios to be computed 
                % we do not compute all efficient portfolios in the
                % efficient frontier (there is a continuum of them)
                % we compute only a subset of them, equally spaced between
                % the minimum variance portfolio and the portfolio with the
                % highest return.

mu=muExR+mean(GER.RF.Ret3Y(ss:se)); %generate mean returns
% generate mean variance efficient frontier
[WeightsMV, MuMeanVariance, StdMeanVariance] = effront(mu, SigmaR, NumPortf);
 
%mean variance for risk fre and tangency portfolio
Mu_bm = zeros(2, 1); 
SD_bm = zeros(2, 1);
Mu_bm(1,1) = mean(GER.RF.Ret3Y(ss:se));
SD_bm(1,1)=0;
Mu_bm(2,1)=mu*WeightsTP;
SD_bm(2,1)=sqrt(WeightsTP'*SigmaR*WeightsTP);
% Plot weights in the tangency portfolio and the efficient frontier
run Figure006
%% ------------------------------------------------------------------------
% Compute the cumulative performances out-of-sample
% -------------------------------------------------------------------------
Ret2 = [GER.LTBond.Ret(ps:pe) GER.Stock.Ret(ps:pe) UK.Stock.Reteu(ps:pe) US.Stock.Reteu(ps:pe)];
PerfR2    = cumsum(Ret2);
Rport     = Ret2*WeightsTP;
PerfPort1 = cumsum(Rport);
PerfRF=cumsum(GER.RiskFree(ps:pe));

% Plot the cumulative performances out-of-sample
run Figure007 
%% ------------------------------------------------------------------------
% Compute the weights and efficient frontier  using a simple
% TVER 
% -------------------------------------------------------------------------

% Graphical Inspection of SOP for Germany and US
run Figure008
% Econometric specification and estimation of SOP
%OLS
XGER=NaN(size(GER.Stock.lp,1),3);
XGER(:,1)=ones;
XGER(:,2)=GER.Stock.lp;
XGER(:,3)=GER.Stock.lpTr;
resGER=ols(GER.Stock.CG3Y(ss:se,:), XGER(ss-36:se-36,:));
prt(resGER)

XUS=NaN(size(GER.Stock.lp,1),3);
XUS(:,1)=ones;
XUS(:,2)=US.Stock.lp;
XUS(:,3)=US.Stock.lpTr;
resUS=ols(US.Stock.CG3Y(ss:se,:), XUS(ss-36:se-36,:));
prt(resUS)

XUK=NaN(size(GER.Stock.lp,1),3);
XUK(:,1)=ones;
XUK(:,2)=UK.Stock.lp;
XUK(:,3)=UK.Stock.lpTr;
resUK=ols(UK.Stock.CG3Y(ss:se,:), XUK(ss-36:se-36,:));
prt(resUK)

%SUR
y1 = GER.Stock.CG3Y(ss:se,:); x11 = GER.Stock.lp(ss-36:se-36,:); x12 = GER.Stock.lpTr(ss-36:se-36,:); % GER
y2 = US.Stock.CG3Y(ss:se,:); x21 = US.Stock.lp(ss-36:se-36,:); x22 = US.Stock.lpTr(ss-36:se-36,:); % US
y3 = UK.Stock.CG3Y(ss:se,:); x31 = UK.Stock.lp(ss-36:se-36,:); x32 = UK.Stock.lpTr(ss-36:se-36,:); % UK
y4 =GER.LTB.Ret3Y(ss:se,:);
nobs = length(GER.Stock.CG3Y(ss:se,:));
iota = ones(nobs,1);
vname1 = strvcat('GER','const','lag P','lag TREND');
vname2 = strvcat('US','const','lag P','lag TREND');
vname3 = strvcat('UK','const','lag P','lag TREND');
vname4 = strvcat('GER YTM 10Y','const');

%vnames= [vname1 vname2 vname3 vname4];
% set up a structure for y in each eqn
y(1).eq = y1; 
y(2).eq = y2; 
y(3).eq = y3; 
y(4).eq= y4;
clear X
% set up a structure for X in each eqn
X(1).eq = [iota x11 x12];
X(2).eq = [iota x21 x22];
X(3).eq = [iota x31 x32];
X(4).eq= [iota ];
SUR=sur(4,y,X);
prt(SUR); 

%BUILD FORECAST 
GER.Stock.Ret3YF=NaN(size(GER.Stock.Index));
UK.Stock.Reteu3YF=NaN(size(GER.Stock.Index));
US.Stock.Reteu3YF=NaN(size(GER.Stock.Index));
GER.Stock.Ret3YCER=NaN(size(GER.Stock.Index));
UK.Stock.Reteu3YCER=NaN(size(GER.Stock.Index));
US.Stock.Reteu3YCER=NaN(size(GER.Stock.Index));
GER.LTB.Ret3YCER=NaN(size(GER.Stock.Index));
for i=ss:pe
%GER.Stock.Ret3YF(i,:)= resGER.beta(1)+ resGER.beta(2)*GER.Stock.lp(i-36,:)+resGER.beta(3)*GER.Stock.lpTr(i-36,:)+GER.Stock.DY3Y(i-36,:);
%UK.Stock.Reteu3YF(i,:)= resUK.beta(1)+ resUK.beta(2)*UK.Stock.lp(i-36,:)+resUK.beta(3)*UK.Stock.lpTr(i-36,:)+UK.Stock.DY3Y(i-36,:);
%US.Stock.Reteu3YF(i,:)= resUS.beta(1)+ resUS.beta(2)*US.Stock.lp(i-36,:)+resUS.beta(3)*US.Stock.lpTr(i-36,:)+US.Stock.DY3Y(i-36,:);
GER.Stock.Ret3YF(i,:)= SUR(1).beta(1)+ SUR(1).beta(2)*GER.Stock.lp(i-36,:)+SUR(1).beta(3)*GER.Stock.lpTr(i-36,:)+GER.Stock.DY3Y(i-36,:);
UK.Stock.Reteu3YF(i,:)= SUR(3).beta(1)+ SUR(3).beta(2)*UK.Stock.lp(i-36,:)+SUR(3).beta(3)*UK.Stock.lpTr(i-36,:)+UK.Stock.DY3Y(i-36,:);
US.Stock.Reteu3YF(i,:)= SUR(2).beta(1)+ SUR(2).beta(2)*US.Stock.lp(i-36,:)+SUR(2).beta(3)*US.Stock.lpTr(i-36,:)+US.Stock.DY3Y(i-36,:);
end
%Forecast based on CER
GER.Stock.Ret3YCER(ss:pe,:)= mean(GER.Stock.Ret3Y(ss:se));
UK.Stock.Reteu3YCER(ss:pe,:)=mean(UK.Stock.Reteu3Y(ss:se));
US.Stock.Reteu3YCER(ss:pe,:)=mean(US.Stock.Reteu3Y(ss:se));
GER.LTB.Ret3YCER(ss:pe,:)= mean(GER.LTB.Ret3Y(ss:se));



%graphic comparison between TVER and CER
run Figure009

%asset allocation with TVER

%Build forecasting errors
GER.LTB.Ret3YFE=GER.LTB.Ret3Y-GER.LTB.Ret3YCER;
GER.Stock.Ret3YFE=GER.Stock.Ret3Y-GER.Stock.Ret3YF;
UK.Stock.Reteu3YFE=UK.Stock.Reteu3Y-UK.Stock.Reteu3YF;
US.Stock.Reteu3YFE=US.Stock.Reteu3Y-US.Stock.Reteu3YF;

Ret3YFE = [GER.LTB.Ret3YFE(ss:se) GER.Stock.Ret3YFE(ss:se)  UK.Stock.Reteu3YFE(ss:se) US.Stock.Reteu3YFE(ss:se)];
muR_TVER    = [GER.LTB.Ret3YCER(pe) GER.Stock.Ret3YF(pe)  UK.Stock.Reteu3YF(pe) US.Stock.Reteu3YF(pe)];
muExR_TVER    = [(GER.LTB.Ret3YCER(pe)-(GER.RF.Ret3Y(pe))) (GER.Stock.Ret3YF(pe)-(GER.RF.Ret3Y(pe)))  (UK.Stock.Reteu3YF(pe)-(GER.RF.Ret3Y(pe))) (US.Stock.Reteu3YF(pe)-(GER.RF.Ret3Y(pe)))];
SigmaR_TVER = cov(Ret3YFE);

% WEIGHTS 
WeightsTP_TVER    = ((SigmaR_TVER^(-1))*muExR_TVER')./(ones(size(SigmaR_TVER,2),1)'*(SigmaR_TVER^(-1)*muExR_TVER'));
%Compute the efficient frontier and plot it 
NumPortf = 20;  % number of efficient portfolios to be computed 
                % we do not compute all efficient portfolios in the
                % efficient frontier (there is a continuum of them)
                % we compute only a subset of them, equally spaced between
                % the minimum variance portfolio and the portfolio with the
                % highest return.

% generate mean variance efficient frontier
[WeightsMV, MuMeanVariance, StdMeanVariance] = effront(muR_TVER, SigmaR_TVER, NumPortf);
 
%mean variance for risk fre and tangency portfolio
Mu_bm_TVER = zeros(2, 1); 
SD_bm_TVER = zeros(2, 1);
Mu_bm_TVER(1,1) = GER.RF.Ret3Y(pe);
SD_bm_TVER(1,1)=0;
Mu_bm_TVER(2,1)=muR_TVER*WeightsTP_TVER;
SD_bm_TVER(2,1)=sqrt(WeightsTP_TVER'*SigmaR_TVER*WeightsTP_TVER);
% Plot weights in the tangency portfolio and the efficient frontier
run Figure010

% Performance 
Rport_TVER     = Ret2*WeightsTP_TVER;
PerfPort_TVER = cumsum(Rport_TVER);

run Figure011










